/********************************************************************************
*                                                                               *
*               D e v i c e   C o n t e x t   B a s e   C l a s s               *
*                                                                               *
*********************************************************************************
* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Library General Public                   *
* License as published by the Free Software Foundation; either                  *
* version 2 of the License, or (at your option) any later version.              *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Library General Public License for more details.                              *
*                                                                               *
* You should have received a copy of the GNU Library General Public             *
* License along with this library; if not, write to the Free                    *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
*********************************************************************************
* $Id: FXDC.h,v 1.2 1999/09/20 22:12:56 jeroen Exp $                           *
********************************************************************************/
#ifndef FXDC_H
#define FXDC_H


class FXApp;
class FXImage;
class FXBitmap;
class FXIcon;
class FXFont;
class FXDrawable;


// Device Context
class FXAPI FXDC : public FXObject {
  FXDECLARE(FXDC)
protected:
  FXApp           *app;       // Application
  FXDrawable      *surface;   // Drawable surface
  FXFont          *font;      // Drawing font
  FXStipplePattern pattern;   // Stipple pattern
  FXBitmap        *stipple;   // Stipple bitmap
  FXImage         *tile;      // Tile image
  FXBitmap        *mask;      // Mask bitmap
  FXColor          fg;        // Foreground color
  FXColor          bg;        // Background color
  FXuint           width;     // Line width
  FXCapStyle       cap;       // Line cap style
  FXJoinStyle      join;      // Line join style
  FXLineStyle      style;     // Line style
  FXFillStyle      fill;      // Fill style
  FXFillRule       rule;      // Fill rule
  FXFunction       rop;       // RasterOp
  FXint            tx;        // Tile dx
  FXint            ty;        // Tile dy
  FXint            cx;        // Clip x
  FXint            cy;        // Clip y
protected:
  FXDC(){}
private:
  FXDC(const FXDC&);
  FXDC &operator=(const FXDC&);
public:
  
  // Construct dummy DC
  FXDC(FXApp* a);
  
  // Get application
  FXApp* getApp() const { return app; }

  // Draw points
  virtual void drawPoint(FXint x,FXint y);
  virtual void drawPoints(const FXPoint* points,FXuint npoints);
  virtual void drawPointsRel(const FXPoint* points,FXuint npoints);

  // Draw lines
  virtual void drawLine(FXint x1,FXint y1,FXint x2,FXint y2);
  virtual void drawLines(const FXPoint* points,FXuint npoints);
  virtual void drawLinesRel(const FXPoint* points,FXuint npoints);
  virtual void drawLineSegments(const FXSegment* segments,FXuint nsegments);
  
  // Draw rectangles
  virtual void drawRectangle(FXint x,FXint y,FXint w,FXint h);
  virtual void drawRectangles(const FXRectangle* rectangles,FXuint nrectangles);

  // Draw arcs
  virtual void drawArc(FXint x,FXint y,FXint w,FXint h,FXint ang1,FXint ang2);
  virtual void drawArcs(const FXArc* arcs,FXuint narcs);

  // Filled rectangles
  virtual void fillRectangle(FXint x,FXint y,FXint w,FXint h);
  virtual void fillRectangles(const FXRectangle* rectangles,FXuint nrectangles);

  // Draw arcs
  virtual void fillArc(FXint x,FXint y,FXint w,FXint h,FXint ang1,FXint ang2);
  virtual void fillArcs(const FXArc* arcs,FXuint narcs);

  // Filled polygon
  virtual void fillPolygon(const FXPoint* points,FXuint npoints);
  virtual void fillConcavePolygon(const FXPoint* points,FXuint npoints);
  virtual void fillComplexPolygon(const FXPoint* points,FXuint npoints);

  // Filled polygon with relative points
  virtual void fillPolygonRel(const FXPoint* points,FXuint npoints);
  virtual void fillConcavePolygonRel(const FXPoint* points,FXuint npoints);
  virtual void fillComplexPolygonRel(const FXPoint* points,FXuint npoints);

  // Draw hashed box
  virtual void drawHashBox(FXint x,FXint y,FXint w,FXint h,FXint b=1);
  
  // Draw area from source
  virtual void drawArea(const FXDrawable* source,FXint sx,FXint sy,FXint sw,FXint sh,FXint dx,FXint dy);

  // Draw image 
  virtual void drawImage(const FXImage* image,FXint dx,FXint dy);

  // Draw bitmap
  virtual void drawBitmap(const FXBitmap* bitmap,FXint dx,FXint dy);
  
  // Draw icon 
  virtual void drawIcon(const FXIcon* icon,FXint dx,FXint dy);
  virtual void drawIconShaded(const FXIcon* icon,FXint dx,FXint dy);

  // Draw string
  virtual void drawText(FXint x,FXint y,const FXchar* string,FXuint length);
  virtual void drawImageText(FXint x,FXint y,const FXchar* string,FXuint length);

  // Set foreground drawing color
  virtual void setForeground(FXColor clr);
  
  // Get foreground drawing color
  FXColor getForeground() const { return fg; }

  // Set background drawing color
  virtual void setBackground(FXColor clr);
  
  // Get background drawing color
  FXColor getBackground() const { return bg; }

  // Set dash pattern
  virtual void setDashes(FXuint dashoffset,const FXchar *dashlist,FXuint n);
  
  // Set line width:- 0 means thinnest/fastest possible
  virtual void setLineWidth(FXuint linewidth=0);
  
  // Get line width
  FXuint getLineWidth() const { return width; }
  
  // Set line cap style
  virtual void setLineCap(FXCapStyle capstyle=CAP_BUTT);
  
  // Get line cap style
  FXCapStyle getLineCap() const { return cap; }
  
  // Set line join style
  virtual void setLineJoin(FXJoinStyle joinstyle=JOIN_MITER);
  
  // Get line join style
  FXJoinStyle getLineJoin() const { return join; }
  
  // Set line style
  virtual void setLineStyle(FXLineStyle linestyle=LINE_SOLID);
  
  // Get line style
  FXLineStyle getLineStyle() const { return style; }
  
  // Set fill style
  virtual void setFillStyle(FXFillStyle fillstyle=FILL_SOLID);
  
  // Get fill style
  FXFillStyle getFillStyle() const { return fill; }
  
  // Set fill rule
  virtual void setFillRule(FXFillRule fillrule=RULE_EVEN_ODD);
  
  // Get fill rule
  FXFillRule getFillRule() const { return rule; }
  
  // Set rasterop function
  virtual void setFunction(FXFunction func=BLT_SRC);
  
  // Get rasterop function
  FXFunction getFunction() const { return rop; }

  // Set the tile image
  virtual void setTile(FXImage* image,FXint dx=0,FXint dy=0);
  
  // Get the tile image
  FXImage *getTile() const { return tile; }
  
  // Set the stipple pattern 
  virtual void setStipple(FXBitmap *bitmap,FXint dx=0,FXint dy=0);
  
  // Get stipple bitmap
  FXBitmap *getStippleBitmap() const { return stipple; }
  
  // Set the stipple pattern 
  virtual void setStipple(FXStipplePattern pat,FXint dx=0,FXint dy=0);
  
  // Get pattern 
  FXStipplePattern getStipplePattern() const { return pattern; }
  
  // Set clip rectangle
  virtual void setClipRectangle(FXint x,FXint y,FXint w,FXint h);
  
  // Clear clipping
  virtual void clearClipRectangle();
  
  // Set clip mask
  virtual void setClipMask(FXBitmap* bitmap,FXint dx=0,FXint dy=0);
  
  // Clear clip mask
  virtual void clearClipMask();
  
  // Set font to draw text with
  virtual void setTextFont(FXFont *fnt);
  
  // Get text font 
  FXFont* getTextFont() const { return font; }
  
  // Clip against child windows
  virtual void clipChildren(FXbool yes);

  // Cleanup
  virtual ~FXDC();
  };
  
    
#endif
