/********************************************************************************
*                                                                               *
*                     D i r e c t o r y   L i s t   W i d g e t                 *
*                                                                               *
*********************************************************************************
* Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Library General Public                   *
* License as published by the Free Software Foundation; either                  *
* version 2 of the License, or (at your option) any later version.              *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Library General Public License for more details.                              *
*                                                                               *
* You should have received a copy of the GNU Library General Public             *
* License along with this library; if not, write to the Free                    *
* Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
*********************************************************************************
* $Id: FXDirList.h,v 1.6 1999/10/27 13:59:07 jeroen Exp $                       *
********************************************************************************/
#ifndef FXDIRLIST_H
#define FXDIRLIST_H



struct FXFileAssoc;                   // File association
class  FXFileDict;                    // Dictionary of file associations



// Directory List options
enum FXDirListStyle {
  DIRLIST_SHOWFILES     = 0x08000000,           // Show files as well as directories
  DIRLIST_SHOWHIDDEN    = 0x10000000            // Show hidden files or directories
  };


// Directory item
class FXAPI FXDirItem : public FXTreeItem {
  FXDECLARE(FXDirItem)
  friend class FXDirList;
protected:
  FXDirItem   *iprev;
  FXDirItem   *inext;
  FXDirItem   *list;
  FXFileAssoc *assoc;
  long         size;
  long         date;
protected:
  FXDirItem():iprev(NULL),inext(NULL),list(NULL),assoc(NULL),date(0){}
protected:
  enum{
    FOLDER      = 256,                // Directory item
    EXECUTABLE  = 512,                // Executable item
    SYMLINK     = 1024,               // Symbolic linked item
    CHARDEV     = 2048,               // Character special item
    BLOCKDEV    = 4096,               // Block special item
    FIFO        = 8192,               // FIFO item
    SOCK        = 16384               // Socket item
    };
public:
  FXDirItem(const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL):FXTreeItem(text,oi,ci,ptr),iprev(NULL),inext(NULL),list(NULL),assoc(NULL),size(0),date(0){}
  FXbool isDirectory() const { return (state&FOLDER)!=0; }
  FXbool isExecutable() const { return (state&EXECUTABLE)!=0; }
  FXbool isSymlink() const { return (state&SYMLINK)!=0; }
  FXbool isChardev() const { return (state&CHARDEV)!=0; }
  FXbool isBlockdev() const { return (state&BLOCKDEV)!=0; }
  FXbool isFifo() const { return (state&FIFO)!=0; }
  FXbool isSocket() const { return (state&SOCK)!=0; }
  FXFileAssoc* getAssoc() const { return assoc; }
  long getSize() const { return size; }
  long getDate() const { return date; }
  };


// Directory Tree List
class FXAPI FXDirList : public FXTreeList {
  FXDECLARE(FXDirList)
protected:
  FXFileDict *associations;           // Association table
  FXString    pattern;                // Pattern of file names
  FXuint      matchmode;              // File wildcard match mode
  FXTimer    *refresh;                // Refresh timer
  FXIcon     *closed_folder;          // Closed folder icon
  FXIcon     *open_folder;            // Open folder icon
  FXIcon     *mini_doc;               // Document icon
  FXIcon     *mini_app;               // Application icon
  FXIcon     *cdromicon;
  FXIcon     *harddiskicon;
  FXIcon     *networkicon;
  FXIcon     *floppyicon;
  FXIcon     *zipdiskicon;
  FXuint      counter;                // Refresh counter
protected:
  FXDirList();
  virtual FXTreeItem* createItem(const FXString& text,FXIcon* oi,FXIcon* ci,void* ptr);
  FXchar *getpath(const FXTreeItem* item,FXchar* pathname) const;
  FXTreeItem* getitem(FXchar* pathname);
  FXbool scanRootDir(FXbool relist);
  FXbool scanSubDir(FXDirItem *par,FXchar *pathname,FXbool relist);
  FXbool listSubDir(FXDirItem *par,FXchar *pathname);
  FXbool listRoots();
private:
  FXDirList(const FXDirList&);
  FXDirList &operator=(const FXDirList&);
public:
  long onRefresh(FXObject*,FXSelector,void*);
  long onBeginDrag(FXObject*,FXSelector,void*);
  long onEndDrag(FXObject*,FXSelector,void*);
  long onDragged(FXObject*,FXSelector,void*);
  long onDNDEnter(FXObject*,FXSelector,void*);
  long onDNDLeave(FXObject*,FXSelector,void*);
  long onDNDMotion(FXObject*,FXSelector,void*);
  long onDNDDrop(FXObject*,FXSelector,void*);
  long onDNDRequest(FXObject*,FXSelector,void*);
  long onSelectionRequest(FXObject*,FXSelector,void* ptr);
  long onItemOpened(FXObject*,FXSelector,void*);
  long onItemClosed(FXObject*,FXSelector,void*);
  long onItemExpanded(FXObject*,FXSelector,void*);
  long onItemCollapsed(FXObject*,FXSelector,void*);
  long onCmdSetValue(FXObject*,FXSelector,void*);
  long onCmdSetStringValue(FXObject*,FXSelector,void*);
  long onCmdGetStringValue(FXObject*,FXSelector,void*);
  long onCmdToggleHidden(FXObject*,FXSelector,void*);
  long onUpdToggleHidden(FXObject*,FXSelector,void*);
  long onCmdShowHidden(FXObject*,FXSelector,void*);
  long onUpdShowHidden(FXObject*,FXSelector,void*);
  long onCmdHideHidden(FXObject*,FXSelector,void*);
  long onUpdHideHidden(FXObject*,FXSelector,void*);
  long onCmdToggleFiles(FXObject*,FXSelector,void*);
  long onUpdToggleFiles(FXObject*,FXSelector,void*);
  long onCmdShowFiles(FXObject*,FXSelector,void*);
  long onUpdShowFiles(FXObject*,FXSelector,void*);
  long onCmdHideFiles(FXObject*,FXSelector,void*);
  long onUpdHideFiles(FXObject*,FXSelector,void*);
  long onCmdSetPattern(FXObject*,FXSelector,void*);
  long onUpdSetPattern(FXObject*,FXSelector,void*);
  long onCmdSortReverse(FXObject*,FXSelector,void*);
  long onUpdSortReverse(FXObject*,FXSelector,void*);
public:
  static FXbool cmpFName(const FXTreeItem* a,const FXTreeItem* b);
  static FXbool cmpRName(const FXTreeItem* pa,const FXTreeItem* pb);
public:
  enum {
    ID_REFRESH=FXTreeList::ID_LAST,
    ID_SHOW_FILES,
    ID_HIDE_FILES,
    ID_TOGGLE_FILES,
    ID_SHOW_HIDDEN,
    ID_HIDE_HIDDEN,
    ID_TOGGLE_HIDDEN,
    ID_SET_PATTERN,
    ID_SORT_REVERSE,
    ID_LAST
    };
public:
  FXDirList(FXComposite *p,FXint nvis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
  virtual void create();
  virtual void detach();
  virtual void destroy();
  FXbool isItemDirectory(const FXTreeItem* item) const;
  FXbool isItemFile(const FXTreeItem* item) const;
  FXbool isItemExecutable(const FXTreeItem* item) const;
  void setCurrentFile(const FXString& file);
  FXString getCurrentFile() const;
  void setDirectory(const FXString& path);
  FXString getDirectory() const;
  FXString getItemFilename(const FXTreeItem* item) const;
  FXString getItemPathname(const FXTreeItem* item) const;
  void setPattern(const FXString& ptrn);
  FXString getPattern() const { return pattern; }
  FXuint getMatchMode() const { return matchmode; }
  void setMatchMode(FXuint mode);
  FXbool showFiles() const;
  void showFiles(FXbool showing);
  FXbool showHiddenFiles() const;
  void showHiddenFiles(FXbool showing);
  void setAssociations(FXFileDict* assoc);
  FXFileDict* getAssociations() const { return associations; }
  virtual void save(FXStream& store) const;
  virtual void load(FXStream& store);
  virtual ~FXDirList();
  };


#endif
